// CaddyService.ts
import { writable, type Writable } from 'svelte/store';

const API_BASE_URL = 'http://localhost:2019'; // Adjust this to your Caddy API endpoint

export interface Upstream {
	address: string;
	num_requests: number;
	fails: number;
}

export interface CAInfo {
	id: string;
	name: string;
	root_common_name: string;
	intermediate_common_name: string;
	root_certificate: string;
	intermediate_certificate: string;
}


export const configStore: Writable<Record<string, any>> = writable({});
export const upstreamsStore: Writable<Upstream[]> = writable([]);

export const CaddyService = {
	async loadConfig(
		config: Record<string, any>,
		contentType: string = 'application/json'
	): Promise<Record<string, any>> {
		try {
			const response = await fetch(`${API_BASE_URL}/load`, {
				method: 'POST',
				headers: {
					'Content-Type': contentType
				},
				body: JSON.stringify(config)
			});
			if (!response.ok) throw new Error('Failed to load configuration');
			return await response.json();
		} catch (error) {
			console.error('Error loading config:', error);
			throw error;
		}
	},

	async stopServer(): Promise<boolean> {
		try {
			const response = await fetch(`${API_BASE_URL}/stop`, { method: 'POST' });
			if (!response.ok) throw new Error('Failed to stop server');
			return true;
		} catch (error) {
			console.error('Error stopping server:', error);
			throw error;
		}
	},

	async getConfig(path: string = ''): Promise<Record<string, any>> {
		try {
			const response = await fetch(`${API_BASE_URL}/config/${path}`);
			if (!response.ok) throw new Error('Failed to get configuration');
			const config = await response.json();
			configStore.set(config);
			return config;
		} catch (error) {
			console.error('Error getting config:', error);
			throw error;
		}
	},

	async updateConfig(
		path: string,
		data: any,
		method: 'POST' | 'PUT' | 'PATCH' | 'DELETE' = 'POST'
	): Promise<Record<string, any>> {
		try {
			const response = await fetch(`${API_BASE_URL}/config/${path}`, {
				method,
				headers: {
					'Content-Type': 'application/json'
				},
				body: JSON.stringify(data)
			});
			if (!response.ok) throw new Error(`Failed to ${method} configuration`);
			await this.getConfig(); // Refresh the config store
			return await response.json();
		} catch (error) {
			console.error(`Error ${method}ing config:`, error);
			throw error;
		}
	},

	async deleteConfig(path: string): Promise<Record<string, any>> {
		return this.updateConfig(path, null, 'DELETE');
	},

	async adaptConfig(config: string, contentType: string): Promise<Record<string, any>> {
		try {
			const response = await fetch(`${API_BASE_URL}/adapt`, {
				method: 'POST',
				headers: {
					'Content-Type': contentType
				},
				body: config
			});
			if (!response.ok) throw new Error('Failed to adapt configuration');
			return await response.json();
		} catch (error) {
			console.error('Error adapting config:', error);
			throw error;
		}
	},

	async getCAInfo(id: string): Promise<CAInfo> {
		try {
			const response = await fetch(`${API_BASE_URL}/pki/ca/${id}`);
			if (!response.ok) throw new Error('Failed to get CA information');
			return await response.json();
		} catch (error) {
			console.error('Error getting CA info:', error);
			throw error;
		}
	},

	async getCACertificates(id: string): Promise<string> {
		try {
			const response = await fetch(`${API_BASE_URL}/pki/ca/${id}/certificates`);
			if (!response.ok) throw new Error('Failed to get CA certificates');
			return await response.text();
		} catch (error) {
			console.error('Error getting CA certificates:', error);
			throw error;
		}
	},

	async getUpstreams(): Promise<Upstream[]> {
		try {
			const response = await fetch(`${API_BASE_URL}/reverse_proxy/upstreams`);
			if (!response.ok) throw new Error('Failed to get upstreams');
			const upstreams = await response.json();
			upstreamsStore.set(upstreams);
			return upstreams;
		} catch (error) {
			console.error('Error getting upstreams:', error);
			throw error;
		}
	}
};
